@@ -1,31 +1,21 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.model import ModelView, ModelSQL, fields, Unique
+from trytond.model import ModelView, ModelSQL, DeactivableMixin, fields, Unique
 from trytond.wizard import Wizard, StateView, Button
-from trytond.transaction import Transaction
 from trytond.pyson import Eval
 from trytond.pool import Pool
 
 
-__all__ = ['BOM', 'BOMInput', 'BOMOutput', 'BOMTree', 'OpenBOMTreeStart',
-    'OpenBOMTreeTree', 'OpenBOMTree']
-
-
-class BOM(ModelSQL, ModelView):
+class BOM(DeactivableMixin, ModelSQL, ModelView):
     "Bill of Material"
     __name__ = 'production.bom'
 
     name = fields.Char('Name', required=True, translate=True)
-    active = fields.Boolean('Active', select=1)
     inputs = fields.One2Many('production.bom.input', 'bom', 'Inputs')
     outputs = fields.One2Many('production.bom.output', 'bom', 'Outputs')
     output_products = fields.Many2Many('production.bom.output',
         'bom', 'product', 'Output Products')
 
-    @staticmethod
-    def default_active():
-        return True
-
     def compute_factor(self, product, quantity, uom):
         '''
         Compute factor for an output product
@@ -45,7 +35,7 @@ class BOM(ModelSQL, ModelView):
             default = {}
         else:
             default = default.copy()
-        default['output_products'] = False
+        default.setdefault('output_products', None)
         return super(BOM, cls).copy(records, default=default)
 
 
@@ -55,10 +45,7 @@ class BOMInput(ModelSQL, ModelView):
 
     bom = fields.Many2One('production.bom', 'BOM', required=True,
         select=1, ondelete='CASCADE')
-    product = fields.Many2One('product.product', 'Product',
-        required=True, domain=[
-            ('type', '!=', 'service'),
-        ])
+    product = fields.Many2One('product.product', 'Product', required=True)
     uom_category = fields.Function(fields.Many2One(
         'product.uom.category', 'Uom Category'), 'on_change_with_uom_category')
     uom = fields.Many2One('product.uom', 'Uom', required=True,
@@ -78,15 +65,17 @@ class BOMInput(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(BOMInput, cls).__setup__()
+        cls.product.domain = [('type', 'in', cls.get_product_types())]
+        cls.__access__.add('bom')
         t = cls.__table__()
         cls._sql_constraints = [
             ('product_bom_uniq', Unique(t, t.product, t.bom),
-                'product_bom_uniq'),
+                'production.msg_product_bom_unique'),
             ]
-        cls._error_messages.update({
-                'product_bom_uniq': 'Product must be unique per BOM.',
-                'recursive_bom': 'You can not create recursive BOMs.',
-                })
+
+    @classmethod
+    def get_product_types(cls):
+        return ['goods', 'assets']
 
     @fields.depends('product', 'uom')
     def on_change_product(self):
@@ -97,7 +86,6 @@ class BOMInput(ModelSQL, ModelView):
                 self.unit_digits = self.product.default_uom.digits
         else:
             self.uom = None
-            self.unit_digits = 2
 
     @fields.depends('product')
     def on_change_with_uom_category(self, name=None):
@@ -253,14 +241,13 @@ class OpenBOMTree(Wizard):
             ])
     tree = StateView('production.bom.tree.open.tree',
         'production.bom_tree_open_tree_view_form', [
-            Button('Change', 'start', 'tryton-go-previous'),
+            Button('Change', 'start', 'tryton-back'),
             Button('Close', 'end', 'tryton-close'),
             ])
 
     def default_start(self, fields):
-        Product = Pool().get('product.product')
         defaults = {}
-        product = Product(Transaction().context['active_id'])
+        product = self.record
         defaults['category'] = product.default_uom.category.id
         if getattr(self.start, 'uom', None):
             defaults['uom'] = self.start.uom.id
