@@ -1,27 +1,30 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+from collections import defaultdict
+from datetime import timedelta
 from decimal import Decimal
+from itertools import chain
 
 from sql import Null
+from sql.conditionals import Coalesce
 
-from trytond.model import ModelView, ModelSQL, Workflow, fields
-from trytond.wizard import Wizard, StateTransition, StateView, Button
-from trytond.pyson import Eval, Bool, If, Id
+from trytond.model import ModelView, ModelSQL, Workflow, fields, dualmethod
+from trytond.pyson import Eval, Bool, If
 from trytond.pool import Pool
 from trytond.transaction import Transaction
 
-from trytond.modules.product import price_digits
-
-__all__ = ['Production', 'AssignFailed', 'Assign']
+from trytond.modules.company.model import employee_field, set_employee
+from trytond.modules.product import price_digits, round_price
+from trytond.modules.stock.shipment import ShipmentAssignMixin
 
 BOM_CHANGES = ['bom', 'product', 'quantity', 'uom', 'warehouse', 'location',
     'company', 'inputs', 'outputs']
 
 
-class Production(Workflow, ModelSQL, ModelView):
+class Production(ShipmentAssignMixin, Workflow, ModelSQL, ModelView):
     "Production"
     __name__ = 'production'
-    _rec_name = 'number'
+    _assign_moves_field = 'inputs'
 
     number = fields.Char('Number', select=True, readonly=True)
     reference = fields.Char('Reference', select=1,
@@ -36,7 +39,7 @@ class Production(Workflow, ModelSQL, ModelView):
         depends=['state'])
     effective_date = fields.Date('Effective Date',
         states={
-            'readonly': Eval('state').in_(['cancel', 'done']),
+            'readonly': Eval('state').in_(['cancelled', 'done']),
             },
         depends=['state'])
     planned_start_date = fields.Date('Planned Start Date',
@@ -47,7 +50,7 @@ class Production(Workflow, ModelSQL, ModelView):
         depends=['state', 'planned_date'])
     effective_start_date = fields.Date('Effective Start Date',
         states={
-            'readonly': Eval('state').in_(['cancel', 'running', 'done']),
+            'readonly': Eval('state').in_(['cancelled', 'running', 'done']),
             },
         depends=['state'])
     company = fields.Many2One('company.company', 'Company', required=True,
@@ -79,7 +82,11 @@ class Production(Workflow, ModelSQL, ModelView):
             ],
         states={
             'readonly': ~Eval('state').in_(['request', 'draft']),
-            })
+            },
+        context={
+            'company': Eval('company', -1),
+            },
+        depends=['company'])
     bom = fields.Many2One('production.bom', 'BOM',
         domain=[
             ('output_products', '=', Eval('product', 0)),
@@ -131,14 +138,21 @@ class Production(Workflow, ModelSQL, ModelView):
         domain=[
             ('shipment', '=', None),
             ('from_location', '=', Eval('location')),
-            ('to_location', 'child_of', [Eval('warehouse')], 'parent'),
+            ['OR',
+                ('to_location', 'child_of', [Eval('warehouse')], 'parent'),
+                ('to_location.waste_warehouses', '=', Eval('warehouse')),
+                ],
             ('company', '=', Eval('company', -1)),
             ],
         states={
-            'readonly': (Eval('state').in_(['done', 'cancel'])
+            'readonly': (Eval('state').in_(['done', 'cancelled'])
                 | ~Eval('warehouse') | ~Eval('location')),
             },
         depends=['warehouse', 'location', 'company'])
+
+    assigned_by = employee_field("Assigned By")
+    run_by = employee_field("Run By")
+    done_by = employee_field("Done By")
     state = fields.Selection([
             ('request', 'Request'),
             ('draft', 'Draft'),
@@ -146,17 +160,22 @@ class Production(Workflow, ModelSQL, ModelView):
             ('assigned', 'Assigned'),
             ('running', 'Running'),
             ('done', 'Done'),
-            ('cancel', 'Canceled'),
+            ('cancelled', 'Cancelled'),
             ], 'State', readonly=True)
+    origin = fields.Reference(
+        "Origin", selection='get_origin', select=True,
+        states={
+            'readonly': ~Eval('state').in_(['request', 'draft']),
+            },
+        depends=['state'])
 
     @classmethod
     def __setup__(cls):
         super(Production, cls).__setup__()
-        cls._error_messages.update({
-                'uneven_costs': ('The costs of the outputs (%(outputs)s) of '
-                    'production "%(production)s" do not match the cost of the '
-                    'production (%(costs)s).')
-                })
+        cls._order = [
+            ('effective_date', 'ASC NULLS LAST'),
+            ('id', 'ASC'),
+            ]
         cls._transitions |= set((
                 ('request', 'draft'),
                 ('draft', 'waiting'),
@@ -167,11 +186,11 @@ class Production(Workflow, ModelSQL, ModelView):
                 ('assigned', 'waiting'),
                 ('waiting', 'waiting'),
                 ('waiting', 'draft'),
-                ('request', 'cancel'),
-                ('draft', 'cancel'),
-                ('waiting', 'cancel'),
-                ('assigned', 'cancel'),
-                ('cancel', 'draft'),
+                ('request', 'cancelled'),
+                ('draft', 'cancelled'),
+                ('waiting', 'cancelled'),
+                ('assigned', 'cancelled'),
+                ('cancelled', 'draft'),
                 ))
         cls._buttons.update({
                 'cancel': {
@@ -181,8 +200,8 @@ class Production(Workflow, ModelSQL, ModelView):
                     },
                 'draft': {
                     'invisible': ~Eval('state').in_(['request', 'waiting',
-                            'cancel']),
-                    'icon': If(Eval('state') == 'cancel',
+                            'cancelled']),
+                    'icon': If(Eval('state') == 'cancelled',
                         'tryton-clear',
                         If(Eval('state') == 'request',
                             'tryton-forward',
@@ -220,6 +239,27 @@ class Production(Workflow, ModelSQL, ModelView):
                 'assign_force': {},
                 })
 
+    def get_rec_name(self, name):
+        items = []
+        if self.number:
+            items.append(self.number)
+        if self.reference:
+            items.append('[%s]' % self.reference)
+        if not items:
+            items.append('(%s)' % self.id)
+        return ' '.join(items)
+
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
+            ('number',) + tuple(clause[1:]),
+            ('reference',) + tuple(clause[1:]),
+            ]
+
     @classmethod
     def __register__(cls, module_name):
         table_h = cls.__table_handler__(module_name)
@@ -239,6 +279,18 @@ class Production(Workflow, ModelSQL, ModelView):
                 where=(table.planned_start_date == Null)
                 & (table.planned_date != Null)))
 
+        # Migration from 5.6: rename state cancel to cancelled
+        cursor.execute(*table.update(
+                [table.state], ['cancelled'],
+                where=table.state == 'cancel'))
+
+    @classmethod
+    def order_effective_date(cls, tables):
+        table, _ = tables[None]
+        return [Coalesce(
+                table.effective_start_date, table.effective_date,
+                table.planned_start_date, table.planned_date)]
+
     @staticmethod
     def default_state():
         return 'draft'
@@ -246,9 +298,7 @@ class Production(Workflow, ModelSQL, ModelView):
     @classmethod
     def default_warehouse(cls):
         Location = Pool().get('stock.location')
-        locations = Location.search(cls.warehouse.domain)
-        if len(locations) == 1:
-            return locations[0].id
+        return Location.get_default_warehouse()
 
     @classmethod
     def default_location(cls):
@@ -262,20 +312,44 @@ class Production(Workflow, ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
-    @fields.depends('planned_date', 'product', 'bom')
-    def on_change_with_planned_start_date(self, pattern=None):
-        if self.planned_date and self.product:
-            if pattern is None:
-                pattern = {}
+    @fields.depends('product', 'bom')
+    def compute_lead_time(self, pattern=None):
+        if pattern is None:
+            pattern = {}
+        if self.product:
             pattern.setdefault('bom', self.bom.id if self.bom else None)
             for line in self.product.lead_times:
                 if line.match(pattern):
-                    if line.lead_time:
-                        return self.planned_date - line.lead_time
-                    else:
-                        return self.planned_date
+                    return line.lead_time or timedelta()
+        return timedelta()
+
+    @fields.depends('planned_date', methods=['compute_lead_time'])
+    def on_change_with_planned_start_date(self, pattern=None):
+        if self.planned_date and self.product:
+            return self.planned_date - self.compute_lead_time()
         return self.planned_date
 
+    @fields.depends(
+        'planned_date', 'planned_start_date', methods=['compute_lead_time'])
+    def on_change_planned_start_date(self, pattern=None):
+        if self.planned_start_date and self.product:
+            planned_date = self.planned_start_date + self.compute_lead_time()
+            if (not self.planned_date
+                    or self.planned_date < planned_date):
+                self.planned_date = planned_date
+
+    @classmethod
+    def _get_origin(cls):
+        'Return list of Model names for origin Reference'
+        return set()
+
+    @classmethod
+    def get_origin(cls):
+        Model = Pool().get('ir.model')
+        get_name = Model.get_name
+        models = cls._get_origin()
+        return [(None, '')] + [(m, get_name(m)) for m in models]
+
     def _move(self, from_location, to_location, company, product, uom,
             quantity):
         Move = Pool().get('stock.move')
@@ -300,24 +374,21 @@ class Production(Workflow, ModelSQL, ModelView):
         move.unit_price_required = move.on_change_with_unit_price_required()
         return move
 
+    @fields.depends(*BOM_CHANGES)
     def explode_bom(self):
         pool = Pool()
         Uom = pool.get('product.uom')
         if not (self.bom and self.product and self.uom):
             return
 
-        if self.warehouse:
-            storage_location = self.warehouse.storage_location
-        else:
-            storage_location = None
-
         factor = self.bom.compute_factor(self.product, self.quantity or 0,
             self.uom)
         inputs = []
         for input_ in self.bom.inputs:
             quantity = input_.compute_quantity(factor)
-            move = self._explode_move_values(storage_location, self.location,
-                self.company, input_, quantity)
+            move = self._explode_move_values(
+                self.picking_location, self.location, self.company,
+                input_, quantity)
             if move:
                 inputs.append(move)
                 quantity = Uom.compute_qty(input_.uom, quantity,
@@ -327,8 +398,9 @@ class Production(Workflow, ModelSQL, ModelView):
         outputs = []
         for output in self.bom.outputs:
             quantity = output.compute_quantity(factor)
-            move = self._explode_move_values(self.location, storage_location,
-                self.company, output, quantity)
+            move = self._explode_move_values(
+                self.location, self.output_location, self.company, output,
+                quantity)
             if move:
                 move.unit_price = Decimal(0)
                 outputs.append(move)
@@ -340,7 +412,7 @@ class Production(Workflow, ModelSQL, ModelView):
         if self.warehouse:
             self.location = self.warehouse.production_location
 
-    @fields.depends(*BOM_CHANGES)
+    @fields.depends('product', 'uom', methods=['explode_bom'])
     def on_change_product(self):
         if self.product:
             category = self.product.default_uom.category
@@ -364,19 +436,19 @@ class Production(Workflow, ModelSQL, ModelView):
             return self.uom.digits
         return 2
 
-    @fields.depends(*BOM_CHANGES)
+    @fields.depends(methods=['explode_bom'])
     def on_change_bom(self):
         self.explode_bom()
 
-    @fields.depends(*BOM_CHANGES)
+    @fields.depends(methods=['explode_bom'])
     def on_change_uom(self):
         self.explode_bom()
 
-    @fields.depends(*BOM_CHANGES)
+    @fields.depends(methods=['explode_bom'])
     def on_change_quantity(self):
         self.explode_bom()
 
-    @ModelView.button_change(*BOM_CHANGES)
+    @ModelView.button_change(methods=['explode_bom'])
     def reset_bom(self):
         self.explode_bom()
 
@@ -388,9 +460,7 @@ class Production(Workflow, ModelSQL, ModelView):
             else:
                 cost_price = input_.product.cost_price
             cost += (Decimal(str(input_.internal_quantity)) * cost_price)
-
-        digits = self.__class__.cost.digits
-        return cost.quantize(Decimal(str(10 ** -digits[1])))
+        return round_price(cost)
 
     @fields.depends('inputs')
     def on_change_with_cost(self):
@@ -411,42 +481,72 @@ class Production(Workflow, ModelSQL, ModelView):
             cost += Decimal(str(quantity)) * product.cost_price
         return cost
 
-    def set_moves(self):
-        storage_location = self.warehouse.storage_location
-        location = self.location
-        company = self.company
+    @dualmethod
+    def set_moves(cls, productions):
+        pool = Pool()
+        Move = pool.get('stock.move')
+        to_save = []
+        for production in productions:
+            location = production.location
+            company = production.company
 
-        if not self.bom:
-            if self.product:
-                move = self._move(location, storage_location, company,
-                    self.product, self.uom, self.quantity)
+            if not production.bom:
+                if production.product:
+                    move = production._move(
+                        location, production.output_location, company,
+                        production.product, production.uom,
+                        production.quantity)
+                    if move:
+                        move.production_output = production
+                        move.unit_price = Decimal(0)
+                        to_save.append(move)
+                continue
+
+            factor = production.bom.compute_factor(
+                production.product, production.quantity, production.uom)
+            for input_ in production.bom.inputs:
+                quantity = input_.compute_quantity(factor)
+                product = input_.product
+                move = production._move(
+                    production.picking_location, location, company, product,
+                    input_.uom, quantity)
                 if move:
-                    move.production_output = self
+                    move.production_input = production
+                    to_save.append(move)
+
+            for output in production.bom.outputs:
+                quantity = output.compute_quantity(factor)
+                product = output.product
+                move = production._move(location, production.output_location,
+                    company, product, output.uom, quantity)
+                if move:
+                    move.production_output = production
                     move.unit_price = Decimal(0)
-                    move.save()
-            self._set_move_planned_date()
-            return
+                    to_save.append(move)
+        Move.save(to_save)
+        cls._set_move_planned_date(productions)
 
-        factor = self.bom.compute_factor(self.product, self.quantity, self.uom)
-        for input_ in self.bom.inputs:
-            quantity = input_.compute_quantity(factor)
-            product = input_.product
-            move = self._move(storage_location, location, company,
-                product, input_.uom, quantity)
-            if move:
-                move.production_input = self
-                move.save()
+    @property
+    def _list_price_context(self):
+        return {
+            'company': self.company.id,
+            }
 
-        for output in self.bom.outputs:
-            quantity = output.compute_quantity(factor)
-            product = output.product
-            move = self._move(location, storage_location, company,
-                product, output.uom, quantity)
-            if move:
-                move.production_output = self
-                move.unit_price = Decimal(0)
-                move.save()
-        self._set_move_planned_date()
+    @classmethod
+    def set_cost_from_moves(cls):
+        pool = Pool()
+        Move = pool.get('stock.move')
+        productions = set()
+        moves = Move.search([
+                ('production_cost_price_updated', '=', True),
+                ('production_input', '!=', None),
+                ],
+            order=[('effective_date', 'ASC')])
+        for move in moves:
+            if move.production_input not in productions:
+                cls.__queue__.set_cost([move.production_input])
+                productions.add(move.production_input)
+        Move.write(moves, {'production_cost_price_updated': False})
 
     @classmethod
     def set_cost(cls, productions):
@@ -454,58 +554,86 @@ class Production(Workflow, ModelSQL, ModelView):
         Uom = pool.get('product.uom')
         Move = pool.get('stock.move')
 
-        digits = Move.unit_price.digits
-        digit = Decimal(str(10 ** -digits[1]))
         moves = []
         for production in productions:
-            if not production.quantity or not production.uom:
-                continue
-            if production.company.currency.is_zero(
-                    production.cost - production.output_cost):
-                continue
-            unit_price = production.cost / Decimal(str(production.quantity))
+            sum_ = Decimal(0)
+            prices = {}
+            cost = production.cost
+
+            input_quantities = defaultdict(Decimal)
+            input_costs = defaultdict(Decimal)
+            for input_ in production.inputs:
+                if input_.cost_price is not None:
+                    cost_price = input_.cost_price
+                else:
+                    cost_price = input_.product.cost_price
+                input_quantities[input_.product] += (
+                    Decimal(str(input_.internal_quantity)))
+                input_costs[input_.product] += (
+                    Decimal(str(input_.internal_quantity)) * cost_price)
+            outputs = []
             for output in production.outputs:
-                if output.product == production.product:
-                    output.unit_price = Uom.compute_price(
-                        production.uom, unit_price, output.uom).quantize(digit)
+                if output.to_location.type == 'lost_found':
+                    continue
+                product = output.product
+                if input_quantities.get(output.product):
+                    cost_price = (
+                        input_costs[product] / input_quantities[product])
+                    unit_price = round_price(Uom.compute_price(
+                            product.default_uom, cost_price, output.uom))
+                    if output.unit_price != unit_price:
+                        output.unit_price = unit_price
+                        moves.append(output)
+                    cost -= min(
+                        unit_price * Decimal(str(output.quantity)), cost)
+                else:
+                    outputs.append(output)
+
+            for output in outputs:
+                product = output.product
+                with Transaction().set_context(production._list_price_context):
+                    list_price = product.list_price_used
+                product_price = (Decimal(str(output.quantity))
+                    * Uom.compute_price(
+                        product.default_uom, list_price, output.uom))
+                prices[output] = product_price
+                sum_ += product_price
+
+            if not sum_ and production.product:
+                prices.clear()
+                for output in outputs:
+                    if output.product == production.product:
+                        quantity = Uom.compute_qty(
+                            output.uom, output.quantity,
+                            output.product.default_uom, round=False)
+                        quantity = Decimal(str(quantity))
+                        prices[output] = quantity
+                        sum_ += quantity
+
+            for output in outputs:
+                if sum_:
+                    ratio = prices.get(output, 0) / sum_
+                else:
+                    ratio = Decimal(1) / len(outputs)
+                quantity = Decimal(str(output.quantity))
+                unit_price = round_price(cost * ratio / quantity)
+                if output.unit_price != unit_price:
+                    output.unit_price = unit_price
                     moves.append(output)
         Move.save(moves)
 
-    @classmethod
-    def validate(cls, productions):
-        super(Production, cls).validate(productions)
-        for production in productions:
-            production.check_cost()
-
-    @property
-    def output_cost(self):
-        cost_price = Decimal(0)
-        for output in self.outputs:
-            cost_price += (Decimal(str(output.quantity)) * output.unit_price)
-        return cost_price
-
-    def check_cost(self):
-        if self.state != 'done':
-            return
-        cost_price = self.output_cost
-        if not self.company.currency.is_zero(self.cost - cost_price):
-            self.raise_user_error('uneven_costs', {
-                    'production': self.rec_name,
-                    'costs': self.cost,
-                    'outputs': cost_price,
-                    })
-
     @classmethod
     def create(cls, vlist):
-        Sequence = Pool().get('ir.sequence')
         Config = Pool().get('production.configuration')
 
         vlist = [x.copy() for x in vlist]
         config = Config(1)
+        default_company = cls.default_company()
         for values in vlist:
             if values.get('number') is None:
-                values['number'] = Sequence.get_id(
-                    config.production_sequence.id)
+                values['number'] = config.get_multivalue(
+                    'production_sequence',
+                    company=values.get('company', default_company)).get()
         productions = super(Production, cls).create(vlist)
         for production in productions:
             production._set_move_planned_date()
@@ -524,30 +652,44 @@ class Production(Workflow, ModelSQL, ModelView):
         else:
             default = default.copy()
         default.setdefault('number', None)
+        default.setdefault('assigned_by')
+        default.setdefault('run_by')
+        default.setdefault('done_by')
         return super(Production, cls).copy(productions, default=default)
 
     def _get_move_planned_date(self):
         "Return the planned dates for input and output moves"
         return self.planned_start_date, self.planned_date
 
-    def _set_move_planned_date(self):
+    @dualmethod
+    def _set_move_planned_date(cls, productions):
         "Set planned date of moves for the shipments"
         pool = Pool()
         Move = pool.get('stock.move')
-        dates = self._get_move_planned_date()
-        input_date, output_date = dates
-        Move.write([m for m in self.inputs
-                if m.state not in ('assigned', 'done', 'cancel')], {
-                'planned_date': input_date,
-                })
-        Move.write([m for m in self.outputs
-                if m.state not in ('assigned', 'done', 'cancel')], {
-                'planned_date': output_date,
-                })
+        to_write = []
+        for production in productions:
+            dates = production._get_move_planned_date()
+            input_date, output_date = dates
+            inputs = [m for m in production.inputs
+                    if m.state not in ('assigned', 'done', 'cancelled')]
+            if inputs:
+                to_write.append(inputs)
+                to_write.append({
+                        'planned_date': input_date,
+                        })
+            outputs = [m for m in production.outputs
+                    if m.state not in ('assigned', 'done', 'cancelled')]
+            if outputs:
+                to_write.append(outputs)
+                to_write.append({
+                        'planned_date': output_date,
+                        })
+        if to_write:
+            Move.write(*to_write)
 
     @classmethod
     @ModelView.button
-    @Workflow.transition('cancel')
+    @Workflow.transition('cancelled')
     def cancel(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
@@ -560,8 +702,16 @@ class Production(Workflow, ModelSQL, ModelView):
     def draft(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
-        Move.draft([m for p in productions
-                for m in p.inputs + p.outputs])
+
+        to_draft, to_delete = [], []
+        for production in productions:
+            for move in chain(production.inputs, production.outputs):
+                if move.state != 'cancelled':
+                    to_draft.append(move)
+                else:
+                    to_delete.append(move)
+        Move.draft(to_draft)
+        Move.delete(to_delete)
 
     @classmethod
     @ModelView.button
@@ -574,12 +724,16 @@ class Production(Workflow, ModelSQL, ModelView):
 
     @classmethod
     @Workflow.transition('assigned')
+    @set_employee('assigned_by')
     def assign(cls, productions):
-        pass
+        pool = Pool()
+        Move = pool.get('stock.move')
+        Move.assign([m for p in productions for m in p.assign_moves])
 
     @classmethod
     @ModelView.button
     @Workflow.transition('running')
+    @set_employee('run_by')
     def run(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
@@ -592,6 +746,7 @@ class Production(Workflow, ModelSQL, ModelView):
     @classmethod
     @ModelView.button
     @Workflow.transition('done')
+    @set_employee('done_by')
     def done(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
@@ -603,79 +758,49 @@ class Production(Workflow, ModelSQL, ModelView):
                 })
 
     @classmethod
-    @ModelView.button_action('production.wizard_assign')
+    @ModelView.button_action('production.wizard_production_assign')
     def assign_wizard(cls, productions):
         pass
 
-    @classmethod
+    @dualmethod
     @ModelView.button
     def assign_try(cls, productions):
         pool = Pool()
         Move = pool.get('stock.move')
-        if Move.assign_try([m for p in productions
-                    for m in p.inputs]):
+        if Move.assign_try(
+                [m for p in productions for m in p.assign_moves]):
             cls.assign(productions)
             return True
         else:
             return False
 
     @classmethod
-    @ModelView.button
-    def assign_force(cls, productions):
+    def _get_reschedule_domain(cls, date):
+        return [
+            ('state', '=', 'waiting'),
+            ('planned_start_date', '<', date),
+            ]
+
+    @classmethod
+    def reschedule(cls, date=None):
         pool = Pool()
-        Move = pool.get('stock.move')
-        Move.assign([m for p in productions for m in p.inputs])
-        cls.assign(productions)
+        Date = pool.get('ir.date')
+        if date is None:
+            date = Date.today()
+        productions = cls.search(cls._get_reschedule_domain(date))
+        for production in productions:
+            production.planned_start_date = date
+            production.on_change_planned_start_date()
+        cls.save(productions)
 
+    @property
+    def picking_location(self):
+        if self.warehouse:
+            return (self.warehouse.production_picking_location
+                or self.warehouse.storage_location)
 
-class AssignFailed(ModelView):
-    'Assign Production'
-    __name__ = 'production.assign.failed'
-
-    moves = fields.Many2Many('stock.move', None, None, 'Moves', readonly=True)
-
-    @staticmethod
-    def default_moves():
-        pool = Pool()
-        Production = pool.get('production')
-        production_id = Transaction().context.get('active_id')
-        if not production_id:
-            return []
-        production = Production(production_id)
-        return [m.id for m in production.inputs if m.state == 'draft']
-
-
-class Assign(Wizard):
-    'Assign Production'
-    __name__ = 'production.assign'
-
-    start = StateTransition()
-    failed = StateView('production.assign.failed',
-        'production.assign_failed_view_form', [
-            Button('Force Assign', 'force', 'tryton-forward',
-                states={
-                    'invisible': ~Id('stock',
-                        'group_stock_force_assignment').in_(
-                        Eval('context', {}).get('groups', [])),
-                    }),
-            Button('OK', 'end', 'tryton-ok', True),
-            ])
-    force = StateTransition()
-
-    def transition_start(self):
-        pool = Pool()
-        Production = pool.get('production')
-
-        if Production.assign_try(
-                [Production(Transaction().context['active_id'])]):
-            return 'end'
-        else:
-            return 'failed'
-
-    def transition_force(self):
-        pool = Pool()
-        Production = pool.get('production')
-
-        Production.assign_force(
-            [Production(Transaction().context['active_id'])])
-        return 'end'
+    @property
+    def output_location(self):
+        if self.warehouse:
+            return (self.warehouse.production_output_location
+                or self.warehouse.storage_location)
